c*********************************************************************
c
c                       =================
c                       SUBROUTINE : CQRT
c                       =================
c
c    This subroutine is the generalized version of the Gordon-Messenger
c    routines in R. G. Gordon and T. Messenger in "Electron-Spin
c    Relaxation in Liquids", L. T. Muus and P. W. Atkins, (1972).
c
c    It calculates the eigenvalues and eigenvectors of the complex
c    symmetric tridiagonal matrix, generated by csqz.f routine.
c
c    Arguments on entry:
c
c        a     : diagonal elements of T-matrix
c        b     : off-diagonal elements of T-matrix
c        amp   : amp as the result of csqz routine
c        nrow  : leading dimension of amp in the calling routine
c        ncol  : following dimension of amp in the calling routine
c        ndim  : actual dimension of matrix a to be diagonalized
c        m     : # of maximum iterations
c
c    on exit:
c
c        a     : eigenvalues of the T-matrix
c        amp   : eigenvectors of the T-matrix
c
c*********************************************************************
c
	subroutine cqrt(a,b,amp,ndim,nrow,ncol,m,tol,eval,ierr)
c
	implicit none
c
        integer i,it,k,kid,ni,ndim,m,n,nrow,ncol,iort,hell,ierr
c
        double precision tol,grass,temp,temp0,rs,br,bi,sig,
     #          zero,one,two,half
c
	complex*16 sh,sm,at,st,stt,u,g,q,r,rc,s,c,sx,cx,cs,
     #          t,tmp,al,am,an,anc,ta,tb,tc,tac,czero,cone,
     #          a(ndim),b(ndim),eval(ndim),amp(nrow,ncol)
c
        parameter (zero=0.0D0,one=1.0D0,two=2.0D0,half=0.5D0,
     #             czero=(0.0D0,0.0D0),cone=(1.0D0,0.0D0))
c
c#####################################################################
c
        ierr=0
	ni=1
        n=ndim
c
	tol=zero
	grass=dfloat(n)
c
	do 200 kid=1,n
 200	  tol=tol+cdabs(a(kid))+two*cdabs(b(kid))
c
	tol=1.D-15*tol/dsqrt(3.0D0*grass)
c
	sh=czero
	k=0
c
	assign 100 to hell
	if(eval(1).ne.czero) assign 101 to hell
	go to hell,(100,101)
c
 100	k=k+1
c
        at=a(n-1)+a(n)
        sm=cdsqrt(at*at-4.0D0*(a(n-1)*a(n)-b(n-1)*b(n-1)))
	st=(at-sm)*half
	stt=(at+sm)*half
	temp=cdabs(a(n)-st)
	temp0=cdabs(a(n)-stt)
	if(temp0.le.temp) st=stt
	go to 102
c
 101	k=k+1
	if(k.ge.m) then
	  ierr=1
          return
        end if
	st=eval(n)-sh
c
 102	if(k.gt.m) then
          ierr=1
          return
        end if
c
	sh=sh+st
c
	do 20 i=1,n
 20	a(i)=a(i)-st
c
	c=cone
        cs=czero
        u=czero
c
	do 10 i=ni,n
          g=a(i)-u
          q=c*a(i)-cs*b(i-1)
   	  if(i.eq.n) go to 10
          sm=cdsqrt(q*q+b(i)*b(i))
          r=sm
	  if(i.ne.ni) b(i-1)=s*r
	  rc=dconjg(r)
	  rs=one/dreal(r*rc)
c
          s=(rc*b(i))*rs
          cs=c*s
          c=(rc*q)*rs
c
          t=g+a(i+1)
          sx=s*s
          u=sx*t
          a(i)=g+u
c
          do 301 iort=1,ndim
	    tmp=amp(iort,i)
	    amp(iort,i)=c*tmp+s*amp(iort,i+1)
	    amp(iort,i+1)=c*amp(iort,i+1)-s*tmp
 301      continue
c
 10	continue
c
	b(n-1)=s*q
	a(n)=g
c
 85	it=n
c
 30	it=it-1
c
        br=dreal(b(it))
        bi=dimag(b(it))
	if((dabs(br)+dabs(bi)).le.tol) go to 40
	if(it-ni) 100,100,30
c
 40	if(n-it-2) 50,60,70
c
 50	a(n)=a(n)+sh
	n=n-1
	k=0
	go to 80
c
 60	al=-b(n-1)
	am=half*(a(n-1)-a(n))
        sm=cdsqrt(al*al+am*am)
	an=sm
	ta=an+am
	tb=an-am
	sig=one
c
	if(cdabs(ta).le.cdabs(tb)) then
	  ta=tb
	  sig=-one
	end if
c
        anc=dconjg(an)
	temp=half/dreal(an*anc)
	c=cdsqrt((anc*ta)*temp)
c
        ta=an*c
        tac=dconjg(ta)
        temp=half/dreal(ta*tac)
        s=sig*(tac*al)*temp
c
        ta=a(n-1)
        tb=a(n)
        tc=b(n-1)
        cx=c*c
        sx=s*s
        cs=c*s
        t=cx-sx
        u=two*(tc*cs)
c
        a(n-1)=ta*cx+tb*sx-u+sh
        a(n)=ta*sx+tb*cx+u+sh
c
        b(n-1)=two*(am*cs)+tc*t
        i=n-1
c
        do 302 iort=1,ndim
	  tmp=amp(iort,i)
	  amp(iort,i)=c*tmp-s*amp(iort,i+1)
	  amp(iort,i+1)=c*amp(iort,i+1)+s*tmp
 302    continue
c
	k=0
c
	n=n-2
	go to 80
c
 70	ni=it+1
c
	go to hell,(100,101)
c
 80	if(n.lt.ni) go to 90
c
 95	if(n-ni-1) 50,60,85
c
 90	if(ni.eq.1) return
c
	ni=1
	go to 95           
c
	end
