c**********************************************************************
c
c                         -------------------
c                         SUBROUTINE : CSCGLC
c                         -------------------
c
c       (C)omplex (S)ymmetric (C)onjugate (G)radient Algorithm 
c
c       This subroutine attempts a solution of an Ax=b type problem 
c       where A is a known sparse complex symmetric matrix, b is a 
c       known vector and the approximate solution vector is obtained 
c       via a simple complex symmetric conjugate gradient procedure.  
c       The algorithm used here is based on algorithm 10.3-1 in 
c       "Matrix Computations", first edition, by G. Golub and C. 
c       Van Loan, Johns Hopkins Univ. Press, 1983.  The "Lanczos" 
c       tridiagonal matrix in constructed using formula 10.2-14 in 
c       the reference above.
c
c       It is assumed in this routine that the right hand vector is 
c       normalized and the solution vector is initialized at the start
c       of the routine.  The matrix and index arrays are passed to 
c       this routine through the inclusion of the file eprmat.inc.
c
c       arguments 
c       ---------
c
c             b     : right hand vector
c             ndim  : number of rows and columns in the matrix 
c             mxcgs : maximum number of conjugate gradient steps allowed
c             cgtol : maximum residual allowed for termination
c             shift : a complex shift value used to avoid extraneous
c                     divisions by zero 
c
c      returns
c      -------
c             x     : approximate solution vector 
c             ter   : b-A*x
c             ndone : number of cg steps executed (positive if 
c                     converged, negative otherwise)
c             terror: true error of the solution vector estimated
c                     Termination of the CG steps is based on this
c                     true error.
c             error : pseudonorm of the residual vector at return
c                     not used in this version
c
c       written by DJS 20-SEP-87
c	modified by SHL 22-JUL-92
c
c       Includes:
c               stddim.inc
c               stdio.inc
c               fnames.inc
c
c       Uses:
c               zdotu.f
c               zaypx.f
c               scmvm.f
c               zaxpy.f
c                               
c**********************************************************************
c
      subroutine cscglc(b,ndim,mxcgs,cgtol,shift,x,ndone,terror)
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'fnames.inc'
c
      integer ndim,mxcgs,ndone
      double precision cgtol,error,terror,tr,ti,zero
      complex*16 x,b,ter,alpha,beta
      dimension x(mxdim),b(mxdim),ter(mxdim)
c
      integer i,nstep
      complex*16 rho1,rho2,czero,shift
      parameter (zero=0.0D0,czero=(0.0D0,0.0D0))
     #           
c
      complex*16 p,r,w
      common /cgdata/ p(mxdim),r(mxdim),w(mxdim)
c
      complex*16 zdotu
      external zdotu
c
c######################################################################
c
c----------------------------------------------------------------------
c    Initialize r and p
c----------------------------------------------------------------------
c
      do 10 i=1,ndim
        r(i)=b(i)
        p(i)=b(i)
 10   continue
c
      rho1=zdotu(r,r,ndim)
c
      write (ludisk1,300)
c
c======================================================================
c     begin loop over CG steps
c======================================================================
c
      nstep=0
 100  nstep=nstep+1
c
c----------------------------------------------------------------------
c     calculate beta 
c----------------------------------------------------------------------
c
      if (nstep.eq.1) then
        beta=czero
      else
        rho2=rho1
        rho1=zdotu(r,r,ndim)
        beta=rho1/rho2
c
      end if
c
c----------------------------------------------------------------------
c     update p  ( p <- r+beta*p )
c----------------------------------------------------------------------
c
      if (nstep.ne.1) call zaypx(r,p,beta,ndim)
c
c----------------------------------------------------------------------
c     calculate w ( w <- A*p )                
c----------------------------------------------------------------------
c
      call scmvm(p,w,ndim)
c
c----------------------------------------------------------------------
c     add diagonal shift term to matrix-vector product
c----------------------------------------------------------------------
c
      call zaxpy(p,w,shift,ndim)
c
c----------------------------------------------------------------------
c     calculate alpha
c----------------------------------------------------------------------
c
      alpha=rho1/zdotu(p,w,ndim)
c
c----------------------------------------------------------------------
c     update x ( x <- x+alpha*p )
c----------------------------------------------------------------------
c
      call zaxpy(p,x,alpha,ndim)
c
c----------------------------------------------------------------------
c     update r ( r <- r-alpha*w )
c----------------------------------------------------------------------
c
      call zaxpy(w,r,-alpha,ndim)
c
c----------------------------------------------------------------------
c     calculate error and terror & check for convergence
c----------------------------------------------------------------------
c
      error=dsqrt(abs(rho1))
c
      if ( (nstep.eq.int(10*(nstep/10))).or.(nstep.eq.mxcgs) ) then
        call scmvm(x,ter,ndim)
        call zaxpy(x,ter,shift,ndim)
        terror=zero
        do 30 i=1,ndim
          ter(i)=b(i)-ter(i)
          tr=dreal(ter(i))
          ti=dimag(ter(i))
          terror=terror+tr*tr+ti*ti
 30     continue
        terror=dsqrt(terror)
        write (ludisk1,310) nstep,error,terror
        if (terror.lt.cgtol) go to 200
        if (terror.lt.1.0D-6) go to 200
c
      end if
c
      if (error.lt.1.D-17) go to 200
c
      if (nstep.lt.mxcgs) go to 100
c
c======================================================================
c     end of loop over CG steps
c======================================================================
c
 200  continue
c
c----------------------------------------------------------------------
c     return error code if not converged
c----------------------------------------------------------------------
c
      if (error.le.cgtol) then
        ndone=nstep-1
      else
        ndone=-nstep+1
      end if
c
c----------------------------------------------------------------------
c     return to calling routine
c----------------------------------------------------------------------
c
      return
c
c----------------------------------------------------------------------
c     format statements
c----------------------------------------------------------------------
c
 300  format(3x,'step',7x,'error (r)',8x,'true error',/)
 310  format(3x,i4,4x,g14.7,4x,g14.7)
      end
