c*********************************************************************
c
c                       =================
c                       SUBROUTINE : CSQZ
c                       =================
c
c    This subroutine is the generalized version of the Gordon-Messenger
c    routines in R. G. Gordon and T. Messenger in "Electron-Spin
c    Relaxation in Liquids", L. T. Muus and P. W. Atkins, (1972).
c
c    The banded complex symmetric matrix is converted into tridiagonal
c    form by a series of Jacobi rotations.  The matrix amp keeps track
c    of those rotations.  To get the eigenvectors, amp should be a unit
c    matrix initially.  After running csqz and cqrt subroutines, the
c    program returns amp as eigenvectors.
c
c    Arguments on entry:
c
c        a     : complex symmetric matrix in band storage format
c                 a(i,1) = i-th diagonal element
c                 a(i,j+1) = j-th element from diagonal in i-th row
c        amp   : unit matrix to be rotated
c        idim  : leading dimension of a in the calling routine
c        jdim  : following dimension of a in the calling routine
c        nrow  : leading dimension of amp in the calling routine
c        ncol  : following dimension of amp in the calling routine
c        n     : actual dimension of matrix a to be diagonalized
c        m     : (half-bandwidth)+1 of matrix a
c
c    on exit:
c
c        a     : a(i,1) = i-th diagonal element of the T-matrix
c                a(i,2) = i-th off-diagonal element of the T-matrix
c        amp   : rotated vector
c
c*********************************************************************
c
	subroutine csqz(a,amp,idim,jdim,nrow,ncol,n,m,sqtol)
c
        implicit none
c
        integer i,j,k,l,m,n,idim,jdim,nrow,ncol,ljm,kid,iort,
     #          mx,nr,nq,nx,iz,ix,ik,ir
c
        double precision rg(2),sqtol,grass,zero,one,two
c
	complex*16 b,g,r,s,sx,c,cx,cs,u,v,temp,tcs,cone,
     #             a(idim,jdim),amp(nrow,ncol)
c
        parameter (zero=0.0D0,one=1.0D0,two=2.0D0,cone=(1.0D0,0.0D0))
c
	equivalence (g,rg(1))
c
c#####################################################################
c
	sqtol=zero
        grass=dfloat(n)
c
	do 101 ljm=1,n
 101	  sqtol=cdabs(a(ljm,1))+sqtol
c
	do 102 kid=2,m
	do 102 ljm=1,n+1-kid
	  grass=grass+two
	  sqtol=two*cdabs(a(ljm,kid))+sqtol
 102    continue
c
	sqtol=1.0D-15*sqtol/dsqrt(grass)
        sqtol=sqtol*sqtol
	mx=m-2
c
	do 20 l=1,mx
	nr=m-l
	nq=nr+1
	nx=n-nr
	if(nx.le.0) go to 20
c
	do 15 k=1,nx
c
	do 25 j=k,nx,nr
c
        if(j.eq.k) then
  	  g=a(j,nq)
          if(rg(1)**2+rg(2)**2.le.sqtol) go to 15
	  b=-a(j,nr)/g
	else
          if(rg(1)**2+rg(2)**2.le.sqtol) go to 15
          b=-a(j-1,nq)/g
        end if
c
        r=cdsqrt(cone+b*b)
        s=cone/r
        c=b*s
c
        sx=s*s
        cx=c*c
        cs=c*s
        tcs=two*cs  
	i=j+nr-1
c
        do 201 iort=1,n
	    temp=amp(iort,i)
	    amp(iort,i)=c*temp-s*amp(iort,i+1)
	    amp(iort,i+1)=c*amp(iort,i+1)+s*temp
 201    continue
c
	u=a(i,1)
	v=a(i+1,1)
	a(i,1)=cx*u-tcs*a(i,2)+sx*v
	a(i+1,1)=sx*u+tcs*a(i,2)+cx*v
	a(i,2)=cs*(u-v)+(cx-sx)*a(i,2)
 	iz=i-1
c
	do 90 ix=j,iz
	  ik=i-ix+1
	  u=a(ix,ik)
	  v=a(ix,ik+1)
	  a(ix,ik)=c*u-s*v
	  a(ix,ik+1)=s*u+c*v
 90     continue
c
        if(j.ne.k) a(j-1,nq)=c*a(j-1,nq)-s*g
c
	ir=min0(nr,n-i)
c
        if(ir.ge.2) then
          do 10 ix=2,ir
	    u=a(i,ix+1)
	    a(i,ix+1)=c*u-s*a(i+1,ix)
 	    a(i+1,ix)=s*u+c*a(i+1,ix)
 10       continue
        end if
c
	if((i+nr-n).lt.0) then
	  g=-s*a(i+1,nq)
	  a(i+1,nq)=c*a(i+1,nq)
        end if
c
 25	continue
c
 15	continue
c
 20	continue
c
	return
	end
